import datetime

from ..utils import find_next_episode
from ..utils import numeric_ranges
from ..format import short_episode
from ..format import format_airdate


def episode_key(now):
    def __episode_key(item):
        season, episode, airdate = item

        if episode is None or airdate is None:
            return 0

        return -((now - airdate).days)

    return __episode_key


def action(ns):
    print ns.term.bold_magenta(u"Next episodes to watch out for")
    print u""

    # prevent excessive time fetching.
    now = datetime.datetime.now()

    episode_sort_key = episode_key(now)

    next_episodes = list()
    all_seen = list()

    for series in ns.series.list_series():
        episodes = ns.series.get_episodes(series)

        if episodes is None:
            print ns.term.bold_red(u"episodes not synced: {0}".format(
                series['series_name']))
            continue

        result = find_next_episode(
            episodes, ns.series.is_episode_watched,
            ignored_seasons=ns.ignored_seasons)

        if result is None:
            all_seen.append(series)
            continue

        next_episode, next_airdate = result
        next_episodes.append((series, next_episode, next_airdate))

    next_episodes = sorted(next_episodes, key=episode_sort_key)

    for series, episode, airdate in next_episodes:
        delta_days = abs((now - airdate).days)

        color = ns.term.bold_white

        if delta_days > ns.relevant_days:
            color = ns.term.white

        print color(u"{2} - {0} {1}".format(
            series['series_name'],
            short_episode(episode),
            format_airdate(episode['first_aired'], now=now)))

    for series in all_seen:
        print ns.term.green(u"{0}: all seen".format(
            series['series_name']))
        continue

    return 0


def setup(parser):
    parser.add_argument(
        '--ignored-seasons', '-i',
        help="Specify a list of seasons to ignore, defaults to '0'",
        default=set([0]),
        type=numeric_ranges,
    )

    parser.add_argument(
        '--relevant-days',
        help=("Specify how many days ago are relevant, these will show up in "
              "a different color, default: 30"),
        default=30,
        type=int,
    )

    parser.set_defaults(action=action)



from ..format import print_series


def action(ns):
    print ns.term.bold_magenta(u"Results from searching online database")
    print u""

    if not ns.synced:
        print ns.term.bold_red("Application not synced")
        return 1

    series = ns.api.getseries(ns.query)

    if not series:
        print ns.term.bold_red("No series matching '{0}'".format(ns.query))
        return 1

    if ns.limit:
        series = series[:ns.limit]

    for s in series:
        print_series(ns.term, s, short_version=True)


def setup(parser):
    parser.add_argument(
        "query",
        metavar="<query>",
    )

    parser.add_argument(
        "--limit",
        metavar="<number>",
        help="Limit the amount of search results displayed, default: 5.",
        type=int,
        default=5)

    parser.set_defaults(action=action)



from ..utils import local_series_finder
from ..utils import with_series
from ..utils import numeric_ranges
from ..utils import find_next_episode


def find_episodes(ns, episodes):
    if ns.next:
        result = find_next_episode(
            episodes, ns.series.is_episode_watched,
            ignored_seasons=ns.ignored_seasons)

        if result is None:
            print ns.term.bold_red(u"no episode is next")
            return

        next_episode, next_airdate = result
        yield next_episode
        return

    for episode in episodes:
        if ns.seasons is not None:
            if episode['season_number'] not in ns.seasons:
                continue

        if ns.episodes is not None:
            if episode['episode_number'] not in ns.episodes:
                continue

        yield episode

    return


@with_series(local_series_finder)
def action(ns, series):
    episodes = ns.series.get_episodes(series)

    changed = 0

    for episode in find_episodes(ns, episodes):
        is_watched = ns.series.is_episode_watched(episode)

        name = u"{0[series_name]} Season {1:02}, Episode {2:02}".format(
            series, episode['season_number'], episode['episode_number'])

        if is_watched and not ns.unmark:
            print ns.term.bold_red(u"already marked: {0}".format(name))
            continue

        if not is_watched and ns.unmark:
            print ns.term.bold_red(u"not marked: {0}".format(name))
            continue

        ns.series.set_episode_watched(episode, (not ns.unmark))

        if not ns.unmark:
            print ns.term.bold_green(u"marked: {0}".format(name))
        else:
            print ns.term.bold_green(u"unmarked: {0}".format(name))

        changed += 1

    if changed == 0:
        print ns.term.bold_red(u"did not change any episodes")
        return 1

    return 0


def setup(parser):
    parser.add_argument(
        "series_id",
        metavar="<name|id>",
        help="The id of the series.",
    )

    parser.add_argument(
        "seasons",
        metavar="seasons",
        nargs='?',
        default=None,
        type=numeric_ranges,
        help="The season of the episode.",
    )

    parser.add_argument(
        "episodes",
        metavar="episodes",
        nargs='?',
        default=None,
        type=numeric_ranges,
        help="The episode number of the season.",
    )

    parser.add_argument(
        "--next",
        default=False,
        help="Mark the next episode not watched.",
        action='store_const',
        const=True,
    )

    parser.add_argument(
        '--ignored-seasons', '-i',
        help="Specify a list of seasons to ignore, defaults to '0'",
        default=set([0]),
        type=numeric_ranges,
    )

    parser.add_argument(
        "--unmark",
        default=False,
        help="The episode number of the season.",
        action='store_const',
        const=True,
    )

    parser.set_defaults(action=action)



from ..utils import local_series_finder
from ..utils import with_series
from ..utils import numeric_ranges
from ..format import print_series


@with_series(local_series_finder)
def action(ns, series):
    print ns.term.bold_magenta(u"Details about series")
    print u""

    print_series(
        ns.term, series,
        focused=ns.seasons,
        focused_episodes=ns.episodes,
        series_dao=ns.series)

    return 0


def setup(parser):
    parser.add_argument(
        "series_id",
        metavar="<name|id>",
        help="The id of the series to add",
    )

    parser.add_argument(
        "seasons",
        nargs="?",
        metavar="[season]",
        type=numeric_ranges,
        help="Filter out the specified season.",
        default=None,
    )

    parser.add_argument(
        "episodes",
        nargs="?",
        metavar="[season]",
        type=numeric_ranges,
        help="Filter out the specified episode.",
        default=None,
    )

    parser.set_defaults(action=action)



from ..utils import write_yaml


def print_configuration_help(ns):
    print ns.term.bold_red(
        "Set a value to the 'api_key' option in {0}".format(ns.config_path))


def action(ns):
    print ns.term.bold_magenta(u"Synchronizing local database")
    print u""

    if not ns.api:
        print ns.term.bold_red("API not configured")
        print_configuration_help(ns)
        return 1

    if not ns.mirrors:
        print ns.term.bold_green("Downloading mirrors to {0}".format(
            ns.mirrors_path))

        with open(ns.mirrors_path, 'w') as fp:
            write_yaml(fp, ns.api.mirrors())

    if not ns.languages:
        with open(ns.languages_path, 'w') as fp:
            write_yaml(fp, ns.api.languages())

    if ns.language:
        for series in ns.series.list_series():
            print ns.term.bold_cyan(
                u"syncing: {0}".format(series['series_name']))

            series, episodes = ns.api.series_all(
                series['id'], ns.language)

            ns.series.set_episodes(series, episodes)

    return 0


def setup(parser):
    parser.set_defaults(action=action)
